#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys
from PIL import Image
from itertools import izip, repeat
from time import time

def get_elapsed_time(tstart = time() * 1000, div = 1):
	return (time() * 1000 - tstart) / 1000 / div

if __name__ == '__main__':

	if len(sys.argv) > 1 and type(sys.argv[1]) == str:
	
		if len(sys.argv) > 2 and type(sys.argv[2]) == str:
			count_method = sys.argv[2]
		else:
			count_method = 'izip'
			
		try:
		
			tstart = time() * 1000
			im = Image.open(sys.argv[1])
			data = im.getdata() # returns a sequence of pixel tuples
			if count_method == 'alt':
				# Use a dictionary to count colors by constructing it with key/value pairs 
				# of str(pixel) = 1
				
				colors = {}
				for pixel in data:
					# Note: Using str(pixel) yields better performance than just using pixel
					colorkey = str(pixel)
					# Note: Just overwriting duplicate keys yields better performance 
					# than first checking colors.has_key(colorkey)
					colors[colorkey] = 1
			else:
				# Use a dictionary to count colors by constructing it with key/value pairs 
				# of pixel = 1
				# izip combines pixel tuples with the infinitely repeating 1 to generate
				# the values for the dictionary constructor.
				
				colors = dict(izip(data, repeat(1)))
			print 'Number of unique', im.mode, 'combinations:', len(colors)
			print 'Elapsed time:', get_elapsed_time(tstart), 'seconds'
		
		except Exception, v:
			print 'Error:', v
			
	else:
	
		if sys.platform == 'win32':
			print 'Usage: image_countcolors.py C:\\path\\to\\imagefile.ext'
		else:
			print 'Usage: image_countcolors.py /path/to/imagefile.ext'
